/*
 * main.c
 */

#include <stdbool.h>
#include <stdio.h>
#include <crc_tbl.h>
#include <driverlib/crc.h>
#include <driverlib/sysctl.h>
#include <inc/hw_memmap.h>

/* Taken from the C2000 crc_tbl.h, since the ARM version doesn't define them (but the linker accepts them) */
#define CRC32_PRIME	  0	/* Poly = 0x04c11db7 */ /* DEFAULT ALGORITHM */
#define CRC16_802_15_4	  1	/* Poly = 0x00001021 */
#define CRC16_ALT	  2	/* Poly = 0x00008005 */
#define CRC32_C           11     /* Poly = 0x1edc6f41 */

extern char text_start[];
extern size_t text_size[];

static void process_crc_table (const CRC_TABLE *const table)
{
	uint32_t rec_index;
	uint32_t actual_crc;
	
	for (rec_index = 0; rec_index < table->num_recs; rec_index++)
	{
		switch (table->recs[rec_index].crc_alg_ID)
		{
		case CRC32_PRIME:
			CRCConfigSet (CCM0_BASE, CRC_CFG_INIT_0 | CRC_CFG_TYPE_P4C11DB7 | CRC_CFG_SIZE_8BIT);
			actual_crc = CRCDataProcess (CCM0_BASE, (uint32_t *) table->recs[rec_index].addr,
					                     table->recs[rec_index].size, false);
			printf ("CRC32_PRIME actual CRC=0x%x, expected CRC=0x%llx\n",
					actual_crc, table->recs[rec_index].crc_value);
			break;

		case CRC16_802_15_4:
			CRCConfigSet (CCM0_BASE, CRC_CFG_INIT_0 | CRC_CFG_TYPE_P1021 | CRC_CFG_SIZE_8BIT);
			actual_crc = CRCDataProcess (CCM0_BASE, (uint32_t *) table->recs[rec_index].addr,
										 table->recs[rec_index].size, false);
			printf ("CRC16_802_15_4 actual CRC=0x%x, expected CRC=0x%llx\n",
					actual_crc, table->recs[rec_index].crc_value);
			break;

		case CRC16_ALT:
			CRCConfigSet (CCM0_BASE, CRC_CFG_INIT_0 | CRC_CFG_TYPE_P8005 | CRC_CFG_SIZE_8BIT);
			actual_crc = CRCDataProcess (CCM0_BASE, (uint32_t *) table->recs[rec_index].addr,
										 table->recs[rec_index].size, false);
			printf ("CRC16_ALT actual CRC=0x%x, expected CRC=0x%llx\n",
					actual_crc, table->recs[rec_index].crc_value);
			break;

		case CRC32_C:
			CRCConfigSet (CCM0_BASE, CRC_CFG_INIT_0 | CRC_CFG_TYPE_P1EDC6F41 | CRC_CFG_SIZE_8BIT);
			actual_crc = CRCDataProcess (CCM0_BASE, (uint32_t *) table->recs[rec_index].addr,
			     		                 table->recs[rec_index].size, false);
			printf ("CRC32_C actual CRC=0x%x, expected CRC=0x%llx\n",
					actual_crc, table->recs[rec_index].crc_value);
			break;

		default:
			printf ("Unknown crc_alg_ID = %u\n", table->recs[rec_index].crc_alg_ID);
			break;
		}
	}
}

extern const CRC_TABLE crc_table_for_intvecs;
extern const CRC_TABLE crc_table_for_text;
extern const CRC_TABLE crc_table_for_const;
extern const CRC_TABLE crc_table_for_cinit;

int main(void) {

	//
	// Enable the CRC module.
	//
	SysCtlPeripheralEnable(SYSCTL_PERIPH_CCM0);
	//
	// Wait for the CRC module to be ready.
	//
	while(!SysCtlPeripheralReady(SYSCTL_PERIPH_CCM0))
	{
	}

	printf ("text_start=0x%p, text_size=0x%x\n", text_start, text_size);
	process_crc_table (&crc_table_for_intvecs);
	process_crc_table (&crc_table_for_text);
	process_crc_table (&crc_table_for_const);
	process_crc_table (&crc_table_for_cinit);

	return 0;
}
